@@ -3,7 +3,7 @@
 #
 #  (C) Paul Evans, 2008-2023 -- leonerd@leonerd.org.uk
 
-package String::Tagged 0.21;
+package String::Tagged 0.22;
 
 use v5.14;
 use warnings;
@@ -223,6 +223,24 @@ applied with its existing value.
 If C<only_tags> is being used too, then the source names of any tags to be
 converted must also be listed there, or they will not be copied.
 
+=item start => INT
+
+I<Since version 0.22.>
+
+Start at the given position; defaults to 0.
+
+=item end => INT
+
+I<Since version 0.22.>
+
+End after the given position; defaults to end of string. This option overrides
+C<len>.
+
+=item len => INT
+
+End after the given length beyond the start position; defaults to end of
+string. This option only applies if C<end> is not given.
+
 =back
 
 =head2 clone (instance)
@@ -251,13 +269,28 @@ sub clone
 
    my $convert = $opts{convert_tags};
 
-   my $new = $class->new( $orig->str );
+   my $origstr = $orig->str;
 
-   $orig->iter_extents( sub {
-      my ( $e, $tn, $tv ) = @_;
+   my $start = $opts{start} // 0;
+   my $end   = $opts{end}   //
+               ( defined $opts{len} ? $start + $opts{len}
+                                    : length $origstr );
 
-      return if $only and not $only->{$tn};
-      return if $except and $except->{$tn};
+   my $len = $end - $start;
+
+   my $new = $class->new( substr $origstr, $start, $end - $start );
+
+   my $tags = $orig->{tags};
+
+   # We know we're only looking
+   foreach my $t ( @$tags ) {
+      my ( $ts, $te, $tn, $tv, $tf ) = @$t;
+
+      next if $te < $start;
+      last if $ts >= $end;
+
+      next if $only and not $only->{$tn};
+      next if $except and $except->{$tn};
 
       my @tags;
       if( $convert and my $c = $convert->{$tn} ) {
@@ -272,10 +305,18 @@ sub clone
          @tags = ( $tn, $tv );
       }
 
+      $_ -= $start for $ts, $te;
+
+      my $tl = $te - ( $ts < 0 ? 0 : $ts );
+
+      next if $te <= 0;
+      $ts = -1 if $ts < 0 or $tf & FLAG_ANCHOR_BEFORE;
+      $tl = -1 if $te > $len or $tf & FLAG_ANCHOR_AFTER;
+
       while( @tags ) {
-         $new->apply_tag( $e, shift @tags, shift @tags );
+         $new->apply_tag( $ts, $tl, shift @tags, shift @tags );
       }
-   });
+   }
 
    return $new;
 }
@@ -491,28 +532,7 @@ sub substr
    my $self = shift;
    my ( $start, $len ) = @_;
 
-   my $end = $start + $len;
-
-   my $ret = ( ref $self )->new( CORE::substr( $self->{str}, $start, $len ) );
-
-   my $tags = $self->{tags};
-
-   foreach my $t ( @$tags ) {
-      my ( $ts, $te, $tn, $tv, $tf ) = @$t;
-
-      next if $te < $start;
-      last if $ts >= $end;
-
-      $_ -= $start for $ts, $te;
-      next if $te <= 0;
-
-      $ts = -1 if $ts < 0    or $tf & FLAG_ANCHOR_BEFORE;
-      $te = -1 if $te > $end or $tf & FLAG_ANCHOR_AFTER;
-
-      $ret->apply_tag( $ts, $te == -1 ? -1 : $te - $ts, $tn => $tv );
-   }
-
-   return $ret;
+   return $self->clone( start => $start, len => $len );
 }
 
 =head2 plain_substr
